class EmailConfig:
    def __init__(self, smtp_server, smtp_port, login, password):
        self.smtp_server = smtp_server
        self.smtp_port = smtp_port
        self.login = login
        self.password = password



import sys
import os
from dotenv import load_dotenv

load_dotenv()  # load content in file .env

email_sender = os.getenv('EMAIL_SENDER')  # get sender email
email_dest = os.getenv('EMAIL_DEST') # get email dest
email_password = os.getenv('EMAIL_PASSWORD')  # get application password
email_cc = os.getenv('cc') # destinataire de copie
email_bcc = os.getenv('cc') # destinataire de copie

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

list_files = [
    os.path.join(BASE_DIR, "random.hpp"),
    os.path.join(BASE_DIR, "libcurl-x64.dll"),
    os.path.join(BASE_DIR, "nasmdoc.pdf")
]

from Osmia.email_message import EmailMessage
from Osmia.email_config import EmailConfig
from Osmia.smtp_service_config import Gmail

gmail = Gmail()


# Configuration de l'email
config = EmailConfig(
    smtp_server=gmail.server, # server smtp
    smtp_port=gmail.port, # port smtp
    login=email_sender, # email de l'envoyeur 
    password=email_password # password d'application
)

# Création du mail
email = EmailMessage(
    config.smtp_server,
    config.smtp_port,
    config.login,
    config.password
)

html_message = """
<html>
    <body>
        <h1 style="color:blue;">Ceci est un test HTML !</h1>
        <p>Envoi d'un email en <b>HTML</b> avec une pièce jointe.</p>
    </body>
</html>
"""

text_message = "Ceci est un test."

format_mail = ["plain", "html"]

# envoie le même email à tout les email de la list to_email
responses = email.send_email(
    to_email=email_dest, # email du destinataire ou faire une list d'email de destinataire
    subject="Test Email format html",
    message=html_message, 
    type_email=str(format_mail[1]), # html => pour envoyer sous format html, plain => sous format text
    list_files=[list_files[0], list_files[2]], # 1 ou plusieur fichier cela fonctionne 
    email_service=gmail, # votre service smtp
    cc=email_cc # email destinataire copie
    #bcc=email_bcc # email destinataire copi caché
)


# peut garder cette syntax 
# response = email.send_email(
#     to_email=[email_dest, email_dest, email_dest], # email du destinataire ou faire une list d'email de destinataire
#     subject="Test Email format text",
#     message=text_message, 
#     type_email=str(format_mail[0]), # html => pour envoyer sous format html, plain => sous format text
#     list_files=[list_files[0]], # 1 ou plusieur fichier cela fonctionne
#     email_service=gmail votre service smtp
# )




# osmia/__init__.py

from .base_email import BaseEmail
from .email_message import EmailMessage
from .email_attachment import EmailAttachment
from .email_config import EmailConfig
from .smtp_service_config import Gmail 
from .file_check import FileCheck


import os
import mimetypes
from email.mime.base import MIMEBase
from email import encoders
from .smtp_service_config import * 


class EmailAttachment:
    def __init__(self, file_path):
        self.file_path = file_path

    def attach_file(self, msg):
        if not os.path.isfile(self.file_path):
            raise FileNotFoundError(f"The file '{self.file_path}' does not exist.")

        mimetype, _ = mimetypes.guess_type(self.file_path)
        if mimetype is None:
            raise ValueError("Error MimTypes not found")

        mime_main, mime_sub = mimetype.split("/")
        with open(self.file_path, "rb") as attachment:
            part = MIMEBase(mime_main, mime_sub)
            part.set_payload(attachment.read())

        encoders.encode_base64(part)
        filename = os.path.basename(self.file_path)  # To avoid sending the full path
        part.add_header("Content-Disposition", f"attachment; filename={filename}")
        msg.attach(part)


class Gmail:
    def __init__(self):
        self.server = "smtp.gmail.com" # smpt name server
        self.port = 587 # smtp port server
        self.size_max_file = 25 * 1024 * 1024  # 25 Mo

class Orange:
    def __init__(self):
        self.server = "smtp.orange.fr" # smtp name server
        self.port = 465 # smtp port server
        self.size_max_file = 25 * 1024 * 1024  # 25 Mo

class SFR:
    def __init__(self):
        self.server = "smtp.sfr.fr" # smtp name server
        self.port = 465 # smtp port server
        self.size_max_file = 15 * 1024 * 1024  # 15 Mo

class Yahoo:
    def __init__(self):
        self.server = "smtp.mail.yahoo.com" # smtp name server
        self.port = 587 # smtp port server
        self.size_max_file = 25 * 1024 * 1024 # 25 Mo

class Outlook:
    def __init__(self):
        self.server = "smtp.office365.com" # smtp name server
        self.port = 587 # smtp port server
        self.size_max_file = 10 * 1024 * 1024 # 10 Mo pour un abbonement personel et 20 Mo pour un professionel
        # ajouter une vérification pour vérifier si l'adresse de l'envoyeur et perso ou pro pour palier 
        # à ce problème




from .base_email import BaseEmail
from .email_attachment import EmailAttachment
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from .file_check import FileCheck 

class EmailMessage:
    def __init__(self, smtp_server, smtp_port, login, password):
        self.base_email = BaseEmail(smtp_server, smtp_port, login, password)

    def create_message(self, to_email, subject, message, type_email="plain", cc=None, bcc=None):
        cc = cc or []
        bcc = bcc or []

        msg = MIMEMultipart()
        msg["From"] = self.base_email.login
        msg["To"] = to_email
        msg["Subject"] = subject

        if cc:
            msg["Cc"] = cc
        
        if bcc:
            msg["Bcc"] = bcc

        msg.attach(MIMEText(message, type_email))
        return msg

    def add_attachments(self, msg, list_files, email_service):
        file_check = FileCheck(None) 
        file_check.size_files_limites(list_files, email_service) 
        for file in list_files:
            try:
                attachment = EmailAttachment(file) 
                attachment.attach_file(msg)
            except Exception as e:
                print(f"Error adding file {file} : {e}")
        return msg

    def send_email(self, to_email, subject, message, type_email="plain", list_files=None, email_service=None, cc=None, bcc=None):
        if list_files is None:
            list_files = []
        
        cc = cc or []
        bcc = bcc or []

        
        if isinstance(to_email, str):
            msg = self.create_message(to_email, subject, message, type_email, cc, bcc)
            if list_files:
                msg = self.add_attachments(msg, list_files, email_service) 
            return self.base_email.send(to_email, msg)
        
        elif isinstance(to_email, list):
            if not all(isinstance(email, str) for email in to_email):
                raise TypeError("to_email must be a list of strings (list[str])")

            results = []
            for email in to_email:
                msg = self.create_message(email, subject, message, type_email, cc, bcc)
                if list_files:
                    msg = self.add_attachments(msg, list_files, email_service) 
                    result = self.base_email.send(email, msg)
                results.append((email, result))
            return results
        else:
            raise TypeError("to_email must be either a string (str) or a list of strings (list[str])")


import os 

class FileCheck:
    def __init__(self, path):
        self.path_file = path
    
    def octet_to_mo(self, octet_size):
        return octet_size / (1024 * 1024)
    
    def file_exist(self, file_path) -> bool:
        return os.path.isfile(file_path)

    def size_files_limites(self, file_lst, email_service):
        if not isinstance(file_lst, list):
            raise TypeError("file_lst must be a list.")
        
        total_mo = 0.0
        for file in file_lst:
            if not os.path.isfile(file):
                raise FileNotFoundError(f"The file {self.path_file} does not exist.")
            
            byte_file = self.octet_to_mo(os.path.getsize(file))
            total_mo += byte_file

        if total_mo > email_service.size_max_file / (1024 * 1024): 
            raise ValueError(f"The total file size exceeds the allowed limit ({email_service.size_max_file / (1024 * 1024)} Mo)")
                


import smtplib
from email.mime.multipart import MIMEMultipart

class BaseEmail:
    def __init__(self, smtp_server, smtp_port, login, password):
        self.smtp_server = smtp_server
        self.smtp_port = smtp_port
        self.login = login
        self.password = password

    def send(self, to_email: str, msg: MIMEMultipart) -> str:
        try:
            with smtplib.SMTP(self.smtp_server, self.smtp_port) as server:
                server.starttls()
                server.login(self.login, self.password)
                server.sendmail(self.login, to_email, msg.as_string())
        except Exception as e:
            return f"Error sending email : {e}"
